<?php
namespace Bazookas\AdminBundle\Filter;
use Symfony\Component\OptionsResolver\OptionsResolver;

/**
 * Helper class to configure a single entity field
 **/
class ListFilterField {

  //field types
  const TYPE_TEXT = 'text';
  const TYPE_NUMBER = 'number';
  const TYPE_BOOLEAN = 'boolean';
  const TYPE_CHOICE = 'choice';
  const TYPE_DATETIME = 'datetime';

  /**
   * @var string the property name of the field to render
   **/
  private $field;

  /**
   * @var string the translation key to use in the label
   **/
  private $label;

  /**
   * @var string the translation key to use in the data attribute of the option
   **/
  private $type;

  /**
   * @var array the allowed operators
   */
  private $operators;

  /**
   * @var Mixed[]
   */
  private $choices;

  public function __construct(array $options = []) {
    $optionsResolver = new OptionsResolver();
    $optionsResolver
      ->setDefaults([
        'field' => null,
        'label' => '',
        'type' => self::TYPE_TEXT,
        'operators' => $this->getDefaultOperators(),
        'choices' => []
      ])
    ;

    $options = $optionsResolver->resolve($options);
    $this->field = $options['field'];
    $this->label = $options['label'];
    $this->type = $options['type'];
    $this->operators = $options['operators'];
    $this->choices = $options['choices'];
  }

  public function getField()
  {
    return $this->field;
  }

  public function setField($field)
  {
    $this->field = $field;

    return $this;
  }

  public function getLabel()
  {
    return $this->label;
  }

  public function setLabel($label)
  {
    $this->label = $label;

    return $this;
  }

  public function getType()
  {
    return $this->type;
  }

  public function setType($type)
  {
    $this->type = $type;

    return $this;
  }

  public function getChoices()
  {
    return $this->choices;
  }

  public function setChoices(array $choices)
  {
    $this->choices = $choices;

    return $this;
  }

  public function setOperators($operators) {
    $this->operators = $operators;
  }

  public function getOperators()
  {
    return $this->operators;
  }

  public static function getDefaultOperators() {
    return array(
      'admin.filter.operator.equals' => ListFilter::OPERATOR_EQUALS,
      'admin.filter.operator.notEquals' => ListFilter::OPERATOR_NOT_EQUALS,
      'admin.filter.operator.like' => ListFilter::OPERATOR_LIKE,
      'admin.filter.operator.notLike' => ListFilter::OPERATOR_NOT_LIKE,
      'admin.filter.operator.greaterThan' => ListFilter::OPERATOR_GREATER_THAN,
      'admin.filter.operator.greaterThanEquals' => ListFilter::OPERATOR_GREATER_THAN_EQUALS,
      'admin.filter.operator.lessThan' => ListFilter::OPERATOR_LESS_THAN,
      'admin.filter.operator.lessThanEquals' => ListFilter::OPERATOR_LESS_THAN_EQUALS,
    );
  }
}
