<?php

namespace Bazookas\CommonBundle\Command\General;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Filesystem\LockHandler;

use Symfony\Component\Console\Input\InputOption;

class ReleaseLockHandlerCommand extends ContainerAwareCommand
{
  private $command = 'command:lock:release';

  protected function configure()
  {
    // TODO check this http://konradpodgorski.com/blog/2013/01/18/how-to-avoid-memory-leaks-in-symfony-2-commands/
    $this
      ->setName($this->command)
      ->setDescription("Forcibly release a command lock that lingered because of errors.")
      ->addOption(
        'command',
        null,
        InputOption::VALUE_REQUIRED,
        'The command to unlock'
      )
      ->addOption(
        'lockPath',
        null,
        InputOption::VALUE_OPTIONAL,
        'The path where the lock file exists, leave blank for default system tmp dir. But note that this might be different for crontab and manual commands.'
      )
    ;
  }

  protected function execute(InputInterface $input, OutputInterface $output)
  {
    // get the options
    $command = $input->getOption('command');
    $lockPath = $input->getOption('lockPath');

    // create the lockhandler
    $commandLock = new LockHandler($command, $lockPath);

    // first lock it, to make sure the file handle is created
    $commandLock->lock();

    // then release the lock and thus make way for other commands
    $commandLock->release();

    // speak into the face of the user and tell their being that it is done and accounted for
    $output->writeln("<info>Done! Unlocked <comment>$command</comment></info>");
  }
}
