<?php
namespace Bazookas\CommonBundle\DependencyInjection\Loader;

use Symfony\Component\DependencyInjection\Loader\FileLoader;
use Symfony\Component\Yaml\Yaml;

/**
 * Use this class to load yaml file and prepend it to the container config
 * Class PrependYamlLoader
 * @package Bazookas\CommonBundle\DependencyInjection\Loader
 */
class PrependYamlLoader extends FileLoader
{

  /**
   * Loads a Yaml file.
   *
   * @param string $file A Yaml file path
   *
   * @return array
   *
   * @throws \InvalidArgumentException When config can't be parsed
   */
  public function load($file, $type = null)
  {
    $path = $this->locator->locate($file);

    $content = Yaml::parse(file_get_contents($path));

    // empty file
    if (empty($content)) {
      $content = [];
    }

    // not an array
    if (!is_array($content)) {
      throw new \InvalidArgumentException(sprintf('The file "%s" must contain a YAML array.', $file));
    }

    foreach ($content as $key => $config) {
      $this->container->prependExtensionConfig($key, $config);
    }

    return $content;
  }

  /**
   * Returns true if this class supports the given resource.
   *
   * @param mixed  $resource A resource
   * @param string $type     The resource type
   *
   * @return Boolean True if this class supports the given resource, false otherwise
   *
   * @api
   */
  public function supports($resource, $type = null)
  {
    return is_string($resource) && 'yml' === pathinfo($resource, PATHINFO_EXTENSION) && (!$type || 'yaml' === $type);
  }
}
