<?php

namespace Bazookas\AdminBundle\PageBuilder;

use Bazookas\AdminBundle\Dashboard\Containers\RowContainer;
use Bazookas\AdminBundle\AdminElements\Elements\BulkFormPanelElement;
use Bazookas\AdminBundle\AdminElements\Elements\HeaderElement;
use Bazookas\AdminBundle\PageBuilder\Base\BaseBulkPageBuilder;
use Bazookas\CommonBundle\Entity\Interfaces\EntityInterface;
use Symfony\Component\HttpFoundation\Request;

class BulkEditPageBuilder extends BaseBulkPageBuilder
{

  public function getEntity()
  {
    return new $this->entityClass();
  }

  public function setEntity(EntityInterface $entity)
  {
    $this->entity = $entity;

    return $this;
  }

  public function handleForm(Request $request) {
    $form = $this->getForm();
    if (empty($form)) {
      throw new \Exception('The form is empty in ' . get_class($this) . ' handleForm' );
    }

    $form->handleRequest($request);

    if ($form->isSubmitted() && $form->isValid()) {
      $data = $form->getData();
      $fields = $request->request->get($form->getName());
      $this->processEdit($data, $fields);

      return true;
    } else {
      $this->form = $form;
      return false;
    }
  }

  public function buildPage(Request $request) {
    $this->elements[] = new RowContainer([
      new HeaderElement([], [
        'label' => 'admin.entities.' . lcfirst($this->getEntityShortClassName()) . '.editLabel'
      ]),
      $this->createPageActionContainer()
    ], [
      'defaultColClass' => 'col-md-6'
    ]);

    $form = $this->getForm();
    if (!empty($form)) {
      $this->elements[] = new BulkFormPanelElement([], [
        'form' => $this->getForm()->createView()
      ]);
    }

  }

  private function processEdit($data, $fields) {
    //get the entities
    $entities = $this->getEntities();

    foreach($entities as $entity) {
      foreach($fields as $field => $value) {
        if ($field != 'submit' && $field != '_token') {
          $setFunction = 'set' . ucfirst($field);
          $getFunction = 'get'.ucfirst($field);
          $entity->$setFunction($data->$getFunction());
        }
      }

      $this->entityManager->persist($entity);
    }

    $this->entityManager->flush();
  }
}
