<?php
namespace Bazookas\APIFrameworkBundle\Exception\Base;

use Bazookas\CommonBundle\Exception\Base\BaseException;
use Exception;
use Bazookas\APIFrameworkBundle\Exception\Interfaces\ApiExceptionInterface;
use Bazookas\CommonBundle\Services\Exception\Interfaces\LoggableExceptionInterface;
use Symfony\Component\HttpFoundation\Response;

abstract class BaseApiException extends BaseException implements ApiExceptionInterface, LoggableExceptionInterface
{

  /**
   * @var bool
   */
  protected $loggable = false;

  /**
   * @var int
   */
  protected $statusCode;

  // Redefine the exception so message isn't optional
  public function __construct($message, $name = ApiExceptionMap::GENERAL_ERROR, $statusCode = Response::HTTP_INTERNAL_SERVER_ERROR, Exception $previous = null) {
    $this->statusCode = $statusCode;

    // make sure everything is assigned properly
    parent::__construct($message, $name, $previous);
  }

  /**
   * @param bool $loggable
   * @return $this
   */
  public function setIsLoggable(bool $loggable) {
    $this->loggable = $loggable;
    return $this;
  }

  /**
   * @return bool
   */
  public function isLoggable() {
    return $this->loggable;
  }

  /**
   * @return int
   */
  public function getHttpStatusCode() {
    return $this->statusCode;
  }

  /**
   * @return string
   */
  public abstract function getLoggingSeverity();

}
