<?php

namespace Bazookas\APIFrameworkBundle\Security\OAuth\GrantTypeExtensions;

use Bazookas\CommonBundle\Exception\NotImplementedException;
use Bazookas\IntegrationsBundle\Entity\Interfaces\FacebookUserInterface;
use Bazookas\IntegrationsBundle\Exception\FacebookIntegrationException;
use Facebook\Exceptions\FacebookAuthenticationException;
use FOS\OAuthServerBundle\Storage\GrantExtensionInterface;
use OAuth2\Model\IOAuth2Client;
use Bazookas\IntegrationsBundle\Services\Facebook\FacebookService;
use Bazookas\IntegrationsBundle\Exception\FacebookTokenExpiredException;
use OAuth2\OAuth2ServerException;
use Symfony\Component\Security\Core\User\UserProviderInterface;

class FacebookOAuthGrantExtension implements GrantExtensionInterface
{
  private $userProvider;
  private $facebookService;
  /**
   * @var bool
   */
  private $allowRegistration;

  public function __construct(UserProviderInterface $userProviderService, FacebookService $facebookService, $register) {
    $this->userProvider = $userProviderService;
    $this->allowRegistration = $register;

    $this->facebookService = $facebookService;
  }

  public function checkGrantExtension(IOAuth2Client $client, array $inputData, array $authHeaders)
  {
    //Verify that the user exists in the database

    //------------------------------------
    //TODO php 5 compatability fix
    $fbid = isset($inputData['fbid']) ? $inputData['fbid'] : 0;

    //TODO enable php7 code
    // $fbid = $inputData['fbid'] ?? 0;
    //------------------------------------

    list($idField, $idGetter) = $this->facebookService->getIdData();
    $user = $this->userProvider->getUserBy(array($idField => $fbid));

    if (empty($user)) {
      if ($this->allowRegistration) {
        if (method_exists($this->userProvider, 'createUser')) {
          $user = $this->userProvider->createUser($fbid, $fbid);
        }
      }
      else {
        throw new OAuth2ServerException(400, 'invalid_grant' ,'No user could be found with the given facebook ID');
      }
    }
    if (!$user instanceof FacebookUserInterface) {
      throw new NotImplementedException('Users used with the facebook grant type must implement Bazookas\IntegrationsBundle\Entity\Interfaces\FacebookUserInterface');
    }

    //verify that the token is valid by retrieving the user's profile
    //------------------------------------
    //TODO php 5 compatability fix
    $accessToken = isset($inputData['token']) ? $inputData['token'] : '';

    //TODO enable php7 code
    // $accessToken = $inputData['token'] ?? '';

    //------------------------------------

    try {
      $longLivedToken = $this->facebookService->getLongLivedToken($accessToken);
      $user->setFacebookAccessToken($longLivedToken);

      $userData = $this->facebookService->getProfile($longLivedToken, 'me');
    } catch (FacebookTokenExpiredException $e) {
      throw new OAuth2ServerException(400, 'invalid_grant' ,'Invalid facebookID and token combination.');
    } catch(FacebookAuthenticationException $e) {
      throw new OAuth2ServerException(400, 'invalid_grant' ,'Invalid facebookID and token combination.');
    } catch (FacebookIntegrationException $e) {
      throw new OAuth2ServerException(400, 'invalid_grant' ,'Invalid facebookID and token combination.');
    }

    if ($userData->getId() == $user->{$idGetter}()) {
      // only update the profile if allowRegistration is true
      if ($this->allowRegistration) {
        $this->facebookService->updateLocalProfile($user, $userData);
      }

      return array(
        'data' => $user
      );
    }
    else {
      throw new OAuth2ServerException(400, 'invalid_grant' ,'Invalid facebookID and token combination.');
    }
  }
}
