## The general template structure

A non-admin web page should use the following twig structure:

Common Layout -> _is extended by_ -> Website layout -> _is embedded by_ -> page template -> _includes_ -> page partials

For example a common website layout template might look like this:

```Twig
{% extends 'BazookasCommonBundle:Layout:layout.html.twig' %}

{% block head %}
  {{ parent() }}

  {% include 'WebsiteBundle:Layout/Partials:_styles.html.twig' %}
{% endblock %}

{% block bodyContent %}
  <div class="content">
    {% block content %}

    {% endblock %}
  </div>
{% endblock %}

{% block bodyFooter %}
  {% include 'WebsiteBundle:Layout/Partials:_scripts.html.twig' %}
{% endblock %}
```

This website layout further fills in some of the basic blocks provided by the common layout. The purpose of the website layout is to include extra `blocks` as needed.

An implementation of a page template might look like this:

```Twig
{% embed 'WebsiteBundle:Layout:layout.html.twig' with {
  metaSettings: {
    'viewport': 'width=device-width, initial-scale=1',
    'keywords': '',
    'description': ''
  },
  ogSettings: {
    'title': 'site.general.title'|trans({}, 'messages'),
    'image': '',
    'description': '',
    'url': url(app.request.get('_route'), app.request.get('_route_params'))
  }
} %}

  {% block content %}

    {% include 'WebsiteBundle:Login/Partials:_somePartial.html.twig' %}

  {% endblock %}

{% endembed %}
```

Your page template can include any partials its wants to. But please note that it is _**NOT**_ possible to further extend your page template. You will have to use a mixture of `use`, `embed`, or `include` to further enhance your page template. 
