<?php

namespace Bazookas\CommonBundle\Validator\Constraints;


use Bazookas\CommonBundle\Enums\Base\BaseEnum;
use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;

class EnumConstraintValidator extends ConstraintValidator
{
  public function validate($value, Constraint $constraint)
  {
    if ($constraint instanceof EnumConstraint) {
      if (empty($constraint->constraintClass)) {
        throw new \Exception('Your ConstantConstraint must define a constraintClass.');
      }

      if (!is_subclass_of($constraint->constraintClass, BaseEnum::class)) {
        throw new \Exception('Your constraintClass must extend \''.BaseEnum::class.'\'');
      }

      if (!$constraint->constraintClass::isValidValue($value)) {
        $constants = $constraint->constraintClass::getConstants();

        $allowedValues = '';
        foreach($constants as $constantName => $constantValue ) {
          $allowedValues .= ', ' . $constantName .' => ' . $constantValue;
        }

        //cut off the first ', '
        $allowedValues = substr($allowedValues, 2, strlen($allowedValues));

        $this->context->buildViolation($constraint->message)
          ->setParameter('{{value}}', $value)
          ->setParameter('{{allowed_values}}', $allowedValues)
          ->addViolation();
      }
    }
  }
}
