<?php

namespace Bazookas\CronBundle\Controller;

use Bazookas\AdminBundle\AdminElements\Elements\Actions\Base\RowActionElement;
use Bazookas\AdminBundle\Controller\Base\BaseAdminListController;
use Bazookas\AdminBundle\Filter\ListFilter;
use Bazookas\AdminBundle\PageBuilder\Interfaces\ListPageBuilderInterface;
use Bazookas\AdminBundle\PageBuilder\ListPageBuilder;
use Bazookas\CommonBundle\Security\Roles;
use Bazookas\CronBundle\Entity\Cron;
use Bazookas\CronBundle\Entity\CronHistory;
use Bazookas\CronBundle\Form\CronAdminType;
use Symfony\Component\HttpFoundation\Request;

class CronAdminController extends BaseAdminListController
{

  const ACTION_TOGGLE_START = 'toggleStart';

  public function __construct() {
    parent::__construct();

    $this->builders[self::ACTION_VIEW] = ListPageBuilder::class;
  }

  protected function getEntityClass()
  {
    return Cron::class;
  }

  protected function hasAccess($action)
  {
    switch ($action) {
      case self::ACTION_EDIT:
      case self::ACTION_ADD:
      case self::ACTION_REMOVE:
      case self::ACTION_VIEW:
      case self::ACTION_LIST:
      case self::ACTION_TOGGLE_START:
        return $this->isGranted(Roles::ROLE_CRON_ADMIN);
      default:
        return false;
    }
  }

  protected function getFormClass($action)
  {
    switch ($action) {
      case self::ACTION_ADD:
      case self::ACTION_EDIT:
        return CronAdminType::class;
      default:
        return null;
    }
  }

  protected function modifyListBuilder(Request $request, ListPageBuilderInterface $builder)
  {
    /**
     * @var ListPageBuilder $builder
     */
    $builder = parent::modifyListBuilder($request, $builder);
    $builder
      ->addField('command')
      ->addField('intervalUnit', [
        'template' => 'BazookasCronBundle:List/Fields:_interval.html.twig',
      ])
      ->addDateTimeField('nextRunOn')
      ->addField('status', [
        'template' => 'BazookasCronBundle:List/Fields:_status.html.twig',
      ])
      ->addField('lastRunStatus', [
        'template' => 'BazookasCronBundle:List/Fields:_status.html.twig',
      ])
      ->addField('failureCount', [
        'template' => 'BazookasCronBundle:List/Fields:_failureCount.html.twig',
      ]);

    //Add a start list action
    $builder->addAction(new RowActionElement([], [
      'template' => 'BazookasCronBundle:List\Partials\ListActions:_toggleStartButton.html.twig',
      'route' => 'bazookas_cron_cron_admin',
    ]));

    return $builder;
  }

  protected function toggleStartAction(Request $request, $action)
  {
    $entityManager = $this->get('doctrine.orm.entity_manager');
    $cronRepo = $entityManager->getRepository(Cron::class);
    $cronRepo->toggleStart($request->get('_id'));

    return $this->redirectToRoute('bazookas_cron_cron_admin', ['_action' => self::ACTION_LIST]);
  }

  protected function modifyViewBuilder(Request $request, ListPageBuilderInterface $builder) {
    /**
     * @var ListPageBuilder $builder
     */
    $builder
      ->setEntityClass(CronHistory::class)
      ->setLimit(50)
      ->setPageAction(self::ACTION_VIEW)

      //Add the fields
      ->addField('runStatus', [
        'template' => 'BazookasCronBundle:List/Fields:_status.html.twig',
      ])
      ->addDateTimeField('runStart')
      ->addDateTimeField('runEnd')
      ->addField('runOutput', [
        'template' => 'BazookasAdminBundle:List\Fields:_collapsible.html.twig'
      ])
    ;

    //Set filtering options
    $builder
      ->addChoiceFilterField('runStatus', [
        'choices' => [
          'admin.entities.cron.enums.status.'.Cron::STATUS_FAILED => Cron::STATUS_FAILED,
          'admin.entities.cron.enums.status.'.Cron::STATUS_LOCKED => Cron::STATUS_LOCKED,
          'admin.entities.cron.enums.status.'.Cron::STATUS_SUCCESS => Cron::STATUS_SUCCESS,
        ]
      ])
    ;

    $builder
      ->addDateTimeFilterField('runStart')
      ->addDateTimeFilterField('runEnd')
    ;

    return $builder;
  }
}
