<?php

namespace Bazookas\ExportBundle\Form;

use Bazookas\ExportBundle\Entity\GenericFileEntity;
use Bazookas\ExportBundle\Enums\CSVDelimiter;
use Bazookas\ExportBundle\Enums\CSVEnclosure;
use Bazookas\ExportBundle\Enums\ImportFileType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\CallbackTransformer;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\Extension\Core\Type\FileType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class ImportFileForm extends AbstractType
{

  /**
   * @param FormBuilderInterface $builder
   * @param array $options
   */
  public function buildForm(FormBuilderInterface $builder, array $options)
  {
    $builder
      ->add('file', FileType::class, [
        'label' => false
      ])
      ->add('fileType', ChoiceType::class, [
        'label' => 'admin.entities.genericFileEntity.fields.fileType',
        'choices' =>$this->prefixEnumsLabels('fileType', ImportFileType::getConstants())
      ])
      ->add('delimiter', ChoiceType::class, [
        'label' => 'admin.entities.genericFileEntity.fields.delimiter',
        'choices' => $this->prefixEnumsLabels('delimiter', CSVDelimiter::getConstants()),
        'conditionallyVisible' => [
          '#import_file_form_fileType' => '$element.val() === "'.ImportFileType::FILE_TYPE_CSV.'"'
        ],
        'required' => false
      ])
      ->add('enclosure', ChoiceType::class, [
        'label' => 'admin.entities.genericFileEntity.fields.enclosure',
        'choices' => $this->createEnclosureChoices(),
        'conditionallyVisible' => [
          '#import_file_form_fileType' => '$element.val() === "'.ImportFileType::FILE_TYPE_CSV.'"'
        ],
        'required' => false
      ])

    ;

    $builder->get('enclosure')->addModelTransformer(new CallbackTransformer(
      //Transform from original to form value
      function ($value) {
        return CSVEnclosure::getNameForValue($value);
      },
      //Transform from form value to original
      function ($value) {
        return CSVEnclosure::isValidName($value) ? constant(CSVEnclosure::class.'::'.$value) : null;
      }
    ));
  }

  /**
   * @param OptionsResolver $resolver
   */
  public function configureOptions(OptionsResolver $resolver)
  {
    $resolver->setDefaults(array(
      'data_class' => GenericFileEntity::class,
      'translation_domain' => 'admin'
    ));
  }

  /**
   * @param $prefix
   * @param $enums
   * @return mixed
   */
  private function prefixEnumsLabels($prefix, $enums) {
    foreach($enums as $label => $value) {
      unset($enums[$label]);
      $enums['admin.entities.genericFileEntity.enums.'.$prefix.'.'.$label] = $value;
    }

    return $enums;
  }

  /**
   * @return array
   */
  private function createEnclosureChoices() {
    $enums = CSVEnclosure::getConstants();
    foreach($enums as $label => $value) {
      unset($enums[$label]);
      $enums['admin.entities.genericFileEntity.enums.enclosure.'.$label] = CSVEnclosure::getNameForValue($value);
    }

    return $enums;
  }
}
